@@ -1,6 +1,6 @@
 /* Pdlzip - LZMA lossless data compressor
    2009-08-14 : Igor Pavlov : Public domain
-   Copyright (C) 2010-2021 Antonio Diaz Diaz.
+   Copyright (C) 2010-2022 Antonio Diaz Diaz.
 
    This program is free software. Redistribution and use in source and
    binary forms, with or without modification, are permitted provided
@@ -21,7 +21,7 @@
    Exit status: 0 for a normal exit, 1 for environmental problems
    (file not found, invalid flags, I/O errors, etc), 2 to indicate a
    corrupt or invalid input file, 3 for an internal consistency error
-   (eg, bug) which caused pdlzip to panic.
+   (e.g., bug) which caused pdlzip to panic.
 */
 
 #define _FILE_OFFSET_BITS 64
@@ -38,9 +38,9 @@
 #include <unistd.h>
 #include <utime.h>
 #include <sys/stat.h>
-#if defined(__MSVCRT__) || defined(__OS2__) || defined(__DJGPP__)
+#if defined __MSVCRT__ || defined __OS2__ || defined __DJGPP__
 #include <io.h>
-#if defined(__MSVCRT__)
+#if defined __MSVCRT__
 #define fchmod(x,y) 0
 #define fchown(x,y,z) 0
 #define SIGHUP SIGTERM
@@ -52,7 +52,7 @@
 #define S_IWOTH 0
 #endif
 #endif
-#if defined(__DJGPP__)
+#if defined __DJGPP__
 #define S_ISSOCK(x) 0
 #define S_ISVTX 0
 #endif
@@ -71,6 +71,11 @@
 #error "Environments where CHAR_BIT != 8 are not supported."
 #endif
 
+#if ( defined  SIZE_MAX &&  SIZE_MAX < UINT_MAX ) || \
+    ( defined SSIZE_MAX && SSIZE_MAX <  INT_MAX )
+#error "Environments where 'size_t' is narrower than 'int' are not supported."
+#endif
+
 int verbosity = 0;
 static void cleanup_and_fail( const int retval );
 static void show_error( const char * const msg, const int errcode,
@@ -80,7 +85,7 @@ static void show_file_error( const char * const filename,
 static void internal_error( const char * const msg );
 
 static const char * const program_name = "pdlzip";
-static const char * const program_year = "2021";
+static const char * const program_year = "2022";
 static const char * invocation_name = "pdlzip";		/* default value */
 
 static const struct { const char * from; const char * to; } known_extensions[] = {
@@ -108,18 +113,18 @@ static void show_help( void )
   {
   printf( "Pdlzip is a permissively licensed implementation of the lzip data\n"
           "compressor, intended for those who can't distribute (or even use) GPL\n"
-          "licensed Free Software. (The name of pdlzip comes from 'public domain\n"
-          "lzip'). Pdlzip is written in C and is (hope)fully compatible with lzip 1.4\n"
-          "or newer.\n"
+          "licensed Free Software. The name of pdlzip comes from 'public domain lzip'.\n"
+          "Pdlzip is written in C and is (hope)fully compatible with lzip 1.4 or newer.\n"
           "\nLzip is a lossless data compressor with a user interface similar to the one\n"
           "of gzip or bzip2. Lzip uses a simplified form of the 'Lempel-Ziv-Markov\n"
-          "chain-Algorithm' (LZMA) stream format, chosen to maximize safety and\n"
-          "interoperability. Lzip can compress about as fast as gzip (lzip -0) or\n"
-          "compress most files more than bzip2 (lzip -9). Decompression speed is\n"
-          "intermediate between gzip and bzip2. Lzip is better than gzip and bzip2 from\n"
-          "a data recovery perspective. Lzip has been designed, written, and tested\n"
-          "with great care to replace gzip and bzip2 as the standard general-purpose\n"
-          "compressed format for unix-like systems.\n"
+          "chain-Algorithm' (LZMA) stream format and provides a 3 factor integrity\n"
+          "checking to maximize interoperability and optimize safety. Lzip can compress\n"
+          "about as fast as gzip (lzip -0) or compress most files more than bzip2\n"
+          "(lzip -9). Decompression speed is intermediate between gzip and bzip2.\n"
+          "Lzip is better than gzip and bzip2 from a data recovery perspective. Lzip\n"
+          "has been designed, written, and tested with great care to replace gzip and\n"
+          "bzip2 as the standard general-purpose compressed format for unix-like\n"
+          "systems.\n"
           "\nPdlzip is also able to decompress legacy lzma-alone (.lzma) files.\n"
           "Lzma-alone is a very bad format; it is essentially a raw LZMA stream.\n"
           "If you keep any lzma-alone files, it is advisable to recompress them to\n"
@@ -160,7 +165,7 @@ static void show_help( void )
           "'tar -xf foo.tar.lz' or 'pdlzip -cd foo.tar.lz | tar -xf -'.\n"
           "\nExit status: 0 for a normal exit, 1 for environmental problems (file\n"
           "not found, invalid flags, I/O errors, etc), 2 to indicate a corrupt or\n"
-          "invalid input file, 3 for an internal consistency error (eg, bug) which\n"
+          "invalid input file, 3 for an internal consistency error (e.g., bug) which\n"
           "caused pdlzip to panic.\n"
           "\nPdlzip includes public domain compression/decompression code from the LZMA\n"
           "SDK (Software Development Kit) written by Igor Pavlov.\n"
@@ -202,8 +207,6 @@ struct Pretty_print
 static void Pp_init( struct Pretty_print * const pp,
                      const char * const filenames[], const int num_filenames )
   {
-  unsigned stdin_name_len;
-  int i;
   pp->name = 0;
   pp->padded_name = 0;
   pp->stdin_name = "(stdin)";
@@ -211,7 +214,8 @@ static void Pp_init( struct Pretty_print * const pp,
   pp->first_post = false;
 
   if( verbosity <= 0 ) return;
-  stdin_name_len = strlen( pp->stdin_name );
+  const unsigned stdin_name_len = strlen( pp->stdin_name );
+  int i;
   for( i = 0; i < num_filenames; ++i )
     {
     const char * const s = filenames[i];
@@ -245,16 +249,14 @@ static void Pp_reset( struct Pretty_print * const pp )
 
 static void Pp_show_msg( struct Pretty_print * const pp, const char * const msg )
   {
-  if( verbosity >= 0 )
+  if( verbosity < 0 ) return;
+  if( pp->first_post )
     {
-    if( pp->first_post )
-      {
-      pp->first_post = false;
-      fputs( pp->padded_name, stderr );
-      if( !msg ) fflush( stderr );
-      }
-    if( msg ) fprintf( stderr, "%s\n", msg );
+    pp->first_post = false;
+    fputs( pp->padded_name, stderr );
+    if( !msg ) fflush( stderr );
     }
+  if( msg ) fprintf( stderr, "%s\n", msg );
   }
 
 
@@ -275,17 +277,53 @@ static void show_header( const unsigned dictionary_size )
   }
 
 
-static unsigned long getnum( const char * const ptr,
+/* separate large numbers >= 100_000 in groups of 3 digits using '_' */
+static const char * format_num3( unsigned long long num )
+  {
+  const char * const si_prefix = "kMGTPEZY";
+  const char * const binary_prefix = "KMGTPEZY";
+  enum { buffers = 8, bufsize = 4 * sizeof (long long) };
+  static char buffer[buffers][bufsize];	/* circle of static buffers for printf */
+  static int current = 0;
+  int i;
+  char * const buf = buffer[current++]; current %= buffers;
+  char * p = buf + bufsize - 1;		/* fill the buffer backwards */
+  *p = 0;	/* terminator */
+  if( num > 1024 )
+    {
+    char prefix = 0;			/* try binary first, then si */
+    for( i = 0; i < 8 && num >= 1024 && num % 1024 == 0; ++i )
+      { num /= 1024; prefix = binary_prefix[i]; }
+    if( prefix ) *(--p) = 'i';
+    else
+      for( i = 0; i < 8 && num >= 1000 && num % 1000 == 0; ++i )
+        { num /= 1000; prefix = si_prefix[i]; }
+    if( prefix ) *(--p) = prefix;
+    }
+  const bool split = num >= 100000;
+
+  for( i = 0; ; )
+    {
+    *(--p) = num % 10 + '0'; num /= 10; if( num == 0 ) break;
+    if( split && ++i >= 3 ) { i = 0; *(--p) = '_'; }
+    }
+  return p;
+  }
+
+
+static unsigned long getnum( const char * const arg,
+                             const char * const option_name,
                              const unsigned long llimit,
                              const unsigned long ulimit )
   {
-  unsigned long result;
   char * tail;
   errno = 0;
-  result = strtoul( ptr, &tail, 0 );
-  if( tail == ptr )
+  unsigned long result = strtoul( arg, &tail, 0 );
+  if( tail == arg )
     {
-    show_error( "Bad or missing numerical argument.", 0, true );
+    if( verbosity >= 0 )
+      fprintf( stderr, "%s: Bad or missing numerical argument in "
+               "option '%s'.\n", program_name, option_name );
     exit( 1 );
     }
 
@@ -308,7 +346,9 @@ static unsigned long getnum( const char * const ptr,
       }
     if( exponent <= 0 )
       {
-      show_error( "Bad multiplier in numerical argument.", 0, true );
+      if( verbosity >= 0 )
+        fprintf( stderr, "%s: Bad multiplier in numerical argument of "
+                 "option '%s'.\n", program_name, option_name );
       exit( 1 );
       }
     for( i = 0; i < exponent; ++i )
@@ -320,21 +360,24 @@ static unsigned long getnum( const char * const ptr,
   if( !errno && ( result < llimit || result > ulimit ) ) errno = ERANGE;
   if( errno )
     {
-    show_error( "Numerical argument out of limits.", 0, false );
+    if( verbosity >= 0 )
+      fprintf( stderr, "%s: Numerical argument out of limits [%s,%s] "
+               "in option '%s'.\n", program_name, format_num3( llimit ),
+               format_num3( ulimit ), option_name );
     exit( 1 );
     }
   return result;
   }
 
 
-static int get_dict_size( const char * const arg )
+static int get_dict_size( const char * const arg, const char * const option_name )
   {
   char * tail;
   const long bits = strtol( arg, &tail, 0 );
   if( bits >= min_dictionary_bits &&
       bits <= max_dictionary_bits_c && *tail == 0 )
     return 1 << bits;
-  return getnum( arg, min_dictionary_size, max_dictionary_size_c );
+  return getnum( arg, option_name, min_dictionary_size, max_dictionary_size_c );
   }
 
 
@@ -408,34 +451,31 @@ static int open_instream( const char * const name, struct stat * const in_statsp
                           const enum Mode program_mode, const int eindex,
                           const bool one_to_one, const bool recompress )
   {
-  int infd = -1;
   if( program_mode == m_compress && !recompress && eindex >= 0 )
     {
     if( verbosity >= 0 )
       fprintf( stderr, "%s: Input file '%s' already has '%s' suffix.\n",
                program_name, name, known_extensions[eindex].from );
+    return -1;
     }
+  int infd = open( name, O_RDONLY | O_BINARY );
+  if( infd < 0 )
+    show_file_error( name, "Can't open input file", errno );
   else
     {
-    infd = open( name, O_RDONLY | O_BINARY );
-    if( infd < 0 )
-      show_file_error( name, "Can't open input file", errno );
-    else
+    const int i = fstat( infd, in_statsp );
+    const mode_t mode = in_statsp->st_mode;
+    const bool can_read = ( i == 0 &&
+                            ( S_ISBLK( mode ) || S_ISCHR( mode ) ||
+                              S_ISFIFO( mode ) || S_ISSOCK( mode ) ) );
+    if( i != 0 || ( !S_ISREG( mode ) && ( !can_read || one_to_one ) ) )
       {
-      const int i = fstat( infd, in_statsp );
-      const mode_t mode = in_statsp->st_mode;
-      const bool can_read = ( i == 0 &&
-                              ( S_ISBLK( mode ) || S_ISCHR( mode ) ||
-                                S_ISFIFO( mode ) || S_ISSOCK( mode ) ) );
-      if( i != 0 || ( !S_ISREG( mode ) && ( !can_read || one_to_one ) ) )
-        {
-        if( verbosity >= 0 )
-          fprintf( stderr, "%s: Input file '%s' is not a regular file%s.\n",
-                   program_name, name, ( can_read && one_to_one ) ?
-                   ",\n        and neither '-c' nor '-o' were specified" : "" );
-        close( infd );
-        infd = -1;
-        }
+      if( verbosity >= 0 )
+        fprintf( stderr, "%s: Input file '%s' is not a regular file%s.\n",
+                 program_name, name, ( can_read && one_to_one ) ?
+                 ",\n        and neither '-c' nor '-o' were specified" : "" );
+      close( infd );
+      infd = -1;
       }
     }
   return infd;
@@ -505,7 +545,7 @@ static bool check_tty_in( const char * const input_filename, const int infd,
       isatty( infd ) )				/* for example /dev/tty */
     { show_file_error( input_filename,
                        "I won't read compressed data from a terminal.", 0 );
-      close( infd ); set_retval( retval, 1 );
+      close( infd ); set_retval( retval, 2 );
       if( program_mode != m_test ) cleanup_and_fail( *retval );
       return false; }
   return true;
@@ -814,12 +854,12 @@ static int decompress( const int infd, struct Pretty_print * const pp,
 
   for( first_member = true; ; first_member = false )
     {
-    int i, size;
+    int i;
     unsigned dictionary_size = 0;		/* keep gcc 3.3.6 happy */
     Lzip_header header;
     if( inSize - inPos < lzma_header_size &&
         !read_inbuf( infd, inBuf, &inPos, &inSize ) ) return 1;
-    size = inSize - inPos;
+    const int size = inSize - inPos;
     for( i = 0; i < size && i < Lh_size; ++i )
       raw_props[i] = header[i] = inBuf[inPos++];
     if( size <= Lh_size )			/* End Of File */
@@ -869,7 +909,6 @@ static int decompress( const int infd, struct Pretty_print * const pp,
       }
     if( lzip_mode )
       {
-      int ds, i;
       if( !Lh_verify_version( header ) )
         {
         if( verbosity >= 0 )
@@ -884,7 +923,7 @@ static int decompress( const int infd, struct Pretty_print * const pp,
           retval = 2; break; }
 
       raw_props[0] = 93;		/* (45 * 2) + (9 * 0) + 3 */
-      ds = dictionary_size;
+      int ds = dictionary_size;
       for( i = 1; i <= 4; ++i ) { raw_props[i] = ds & 0xFF; ds >>= 8; }
       }
 
@@ -913,8 +952,8 @@ static int decompress( const int infd, struct Pretty_print * const pp,
 CRC32 crc32;
 
 
-/* Returns the number of bytes really read.
-   If (returned value < size) and (errno == 0), means EOF was reached.
+/* Return the number of bytes really read.
+   If (value returned < size) and (errno == 0), means EOF was reached.
 */
 int readblock( const int fd, uint8_t * const buf, const int size )
   {
@@ -932,8 +971,8 @@ int readblock( const int fd, uint8_t * const buf, const int size )
   }
 
 
-/* Returns the number of bytes really written.
-   If (returned value < size), it is always an error.
+/* Return the number of bytes really written.
+   If (value returned < size), it is always an error.
 */
 int writeblock( const int fd, const uint8_t * const buf, const int size )
   {
@@ -1000,23 +1039,15 @@ int main( const int argc, const char * const argv[] )
     { 1 << 25, 273 } };		/* -9 */
   struct Lzma_options encoder_options = option_mapping[6];  /* default = "-6" */
   const char * default_output_filename = "";
-  static struct Arg_parser parser;	/* static because valgrind complains */
-  static struct Pretty_print pp;	/* and memory management in C sucks */
-  static const char ** filenames = 0;
-  int num_filenames = 0;
   enum Mode program_mode = m_compress;
-  int argind = 0;
-  int failed_tests = 0;
-  int retval = 0;
   int i;
-  bool filenames_given = false;
   bool force = false;
   bool ignore_trailing = true;
   bool keep_input_files = false;
   bool loose_trailing = false;
   bool recompress = false;
-  bool stdin_used = false;
   bool to_stdout = false;
+  if( argc > 0 ) invocation_name = argv[0];
 
   enum { opt_lt = 256 };
   const struct ap_Option options[] =
@@ -1051,19 +1082,22 @@ int main( const int argc, const char * const argv[] )
     { opt_lt, "loose-trailing", ap_no  },
     {  0, 0,                    ap_no  } };
 
-  if( argc > 0 ) invocation_name = argv[0];
   CRC32_init();
 
+  /* static because valgrind complains and memory management in C sucks */
+  static struct Arg_parser parser;
   if( !ap_init( &parser, argc, argv, options, 0 ) )
     { show_error( mem_msg, 0, false ); return 1; }
   if( ap_error( &parser ) )				/* bad option */
     { show_error( ap_error( &parser ), 0, true ); return 1; }
 
+  int argind = 0;
   for( ; argind < ap_arguments( &parser ); ++argind )
     {
     const int code = ap_code( &parser, argind );
-    const char * const arg = ap_argument( &parser, argind );
     if( !code ) break;					/* no more options */
+    const char * const pn = ap_parsed_name( &parser, argind );
+    const char * const arg = ap_argument( &parser, argind );
     switch( code )
       {
       case '0': case '1': case '2': case '3': case '4':
@@ -1078,12 +1112,12 @@ int main( const int argc, const char * const argv[] )
       case 'h': show_help(); return 0;
       case 'k': keep_input_files = true; break;
       case 'm': encoder_options.match_len_limit =
-                  getnum( arg, min_match_len_limit, max_match_len ); break;
+                  getnum( arg, pn, min_match_len_limit, max_match_len ); break;
       case 'n': break;
       case 'o': if( strcmp( arg, "-" ) == 0 ) to_stdout = true;
                 else { default_output_filename = arg; } break;
       case 'q': verbosity = -1; break;
-      case 's': encoder_options.dictionary_size = get_dict_size( arg );
+      case 's': encoder_options.dictionary_size = get_dict_size( arg, pn );
                 break;
       case 'S': break;
       case 't': set_mode( &program_mode, m_test ); break;
@@ -1094,15 +1128,17 @@ int main( const int argc, const char * const argv[] )
       }
     } /* end process options */
 
-#if defined(__MSVCRT__) || defined(__OS2__) || defined(__DJGPP__)
+#if defined __MSVCRT__ || defined __OS2__ || defined __DJGPP__
   setmode( STDIN_FILENO, O_BINARY );
   setmode( STDOUT_FILENO, O_BINARY );
 #endif
 
-  num_filenames = max( 1, ap_arguments( &parser ) - argind );
+  static const char ** filenames = 0;
+  int num_filenames = max( 1, ap_arguments( &parser ) - argind );
   filenames = resize_buffer( filenames, num_filenames * sizeof filenames[0] );
   filenames[0] = "-";
 
+  bool filenames_given = false;
   for( i = 0; argind + i < ap_arguments( &parser ); ++i )
     {
     filenames[i] = ap_argument( &parser, argind + i );
@@ -1123,16 +1159,18 @@ int main( const int argc, const char * const argv[] )
   if( !to_stdout && program_mode != m_test && ( filenames_given || to_file ) )
     set_signals( signal_handler );
 
+  static struct Pretty_print pp;
   Pp_init( &pp, filenames, num_filenames );
 
+  int failed_tests = 0;
+  int retval = 0;
   const bool one_to_one = !to_stdout && program_mode != m_test && !to_file;
+  bool stdin_used = false;
   for( i = 0; i < num_filenames; ++i )
     {
     const char * input_filename = "";
     int infd;
-    int tmp;
     struct stat in_stats;
-    const struct stat * in_statsp;
 
     Pp_set_name( &pp, filenames[i] );
     if( strcmp( filenames[i], "-" ) == 0 )
@@ -1174,7 +1212,9 @@ int main( const int argc, const char * const argv[] )
         return 1;	/* check tty only once and don't try to delete a tty */
       }
 
-    in_statsp = ( input_filename[0] && one_to_one ) ? &in_stats : 0;
+    const struct stat * const in_statsp =
+      ( input_filename[0] && one_to_one ) ? &in_stats : 0;
+    int tmp;
     if( program_mode == m_compress )
       tmp = compress( &encoder_options, &pp, infd );
     else
